@@ -3,12 +3,12 @@ import logging
 import os
 import random
 import select
+import sys
 import threading
 from collections import deque
 import collections.abc
 from contextlib import contextmanager, closing
 
-import outcome
 from contextvars import copy_context
 from math import inf
 from time import monotonic
@@ -16,11 +16,9 @@ from time import monotonic
 from sniffio import current_async_library_cvar
 
 import attr
-from async_generator import (
-    async_generator, yield_, asynccontextmanager, isasyncgen
-)
+from async_generator import isasyncgen
 from sortedcontainers import SortedDict
-from outcome import Error, Value
+from outcome import Error, Value, capture
 
 from . import _public
 from ._entry_queue import EntryQueue, TrioToken
@@ -65,6 +63,34 @@ _r = random.Random()
 INSTRUMENT_LOGGER = logging.getLogger("trio.abc.Instrument")
 
 
+# On 3.7+, Context.run() is implemented in C and doesn't show up in
+# tracebacks. On 3.6 and earlier, we use the contextvars backport, which is
+# currently implemented in Python and adds 1 frame to tracebacks. So this
+# function is a super-overkill version of "0 if sys.version_info >= (3, 7)
+# else 1". But if Context.run ever changes, we'll be ready!
+#
+# This can all be removed once we drop support for 3.6.
+def _count_context_run_tb_frames():
+    def function_with_unique_name_xyzzy():
+        1 / 0
+
+    ctx = copy_context()
+    try:
+        ctx.run(function_with_unique_name_xyzzy)
+    except ZeroDivisionError as exc:
+        tb = exc.__traceback__
+        # Skip the frame where we caught it
+        tb = tb.tb_next
+        count = 0
+        while tb.tb_frame.f_code.co_name != "function_with_unique_name_xyzzy":
+            tb = tb.tb_next
+            count += 1
+        return count
+
+
+CONTEXT_RUN_TB_FRAMES = _count_context_run_tb_frames()
+
+
 @attr.s(frozen=True)
 class SystemClock:
     # Add a large random offset to our clock to ensure that if people
@@ -90,12 +116,23 @@ class SystemClock:
 @attr.s(cmp=False, hash=False, repr=False)
 class CancelScope:
     _tasks = attr.ib(default=attr.Factory(set))
+    _scope_task = attr.ib(default=None)
     _effective_deadline = attr.ib(default=inf)
     _deadline = attr.ib(default=inf)
     _shield = attr.ib(default=False)
     cancel_called = attr.ib(default=False)
     cancelled_caught = attr.ib(default=False)
 
+    @staticmethod
+    def _create(deadline, shield):
+        task = _core.current_task()
+        scope = CancelScope()
+        scope._scope_task = task
+        scope._add_task(task)
+        scope.deadline = deadline
+        scope.shield = shield
+        return scope
+
     def __repr__(self):
         return "<cancel scope object at {:#x}>".format(id(self))
 
@@ -184,24 +221,53 @@ class CancelScope:
             return None
         return exc
 
+    def _close(self, exc):
+        self._remove_task(self._scope_task)
+        if exc is not None:
+            filtered_exc = MultiError.filter(self._exc_filter, exc)
+            return filtered_exc
+
+
+# We explicitly avoid @contextmanager since it adds extraneous stack frames
+# to exceptions.
+@attr.s
+class CancelScopeManager:
+
+    _deadline = attr.ib(default=inf)
+    _shield = attr.ib(default=False)
+
+    @enable_ki_protection
+    def __enter__(self):
+        self._scope = CancelScope._create(self._deadline, self._shield)
+        return self._scope
+
+    @enable_ki_protection
+    def __exit__(self, etype, exc, tb):
+        # Tracebacks show the 'raise' line below out of context, so let's give
+        # this variable a name that makes sense out of context.
+        remaining_error_after_cancel_scope = self._scope._close(exc)
+        if remaining_error_after_cancel_scope is None:
+            return True
+        elif remaining_error_after_cancel_scope is exc:
+            return False
+        else:
+            # Copied verbatim from MultiErrorCatcher.  Python doesn't
+            # allow us to encapsulate this __context__ fixup.
+            old_context = remaining_error_after_cancel_scope.__context__
+            try:
+                raise remaining_error_after_cancel_scope
+            finally:
+                _, value, _ = sys.exc_info()
+                assert value is remaining_error_after_cancel_scope
+                value.__context__ = old_context
+
 
-@contextmanager
-@enable_ki_protection
 def open_cancel_scope(*, deadline=inf, shield=False):
     """Returns a context manager which creates a new cancellation scope.
 
     """
 
-    task = _core.current_task()
-    scope = CancelScope()
-    scope._add_task(task)
-    scope.deadline = deadline
-    scope.shield = shield
-    try:
-        with MultiError.catch(scope._exc_filter):
-            yield scope
-    finally:
-        scope._remove_task(task)
+    return CancelScopeManager(deadline, shield)
 
 
 ################################################################
@@ -295,57 +361,54 @@ class _TaskStatus:
         self._old_nursery._check_nursery_closed()
 
 
-@asynccontextmanager
-@async_generator
-@enable_ki_protection
-async def open_nursery():
-    """Returns an async context manager which creates a new nursery.
+class NurseryManager:
+    """Nursery context manager.
 
-    This context manager's ``__aenter__`` method executes synchronously. Its
-    ``__aexit__`` method blocks until all child tasks have exited.
+    Note we explicitly avoid @asynccontextmanager and @async_generator
+    since they add a lot of extraneous stack frames to exceptions, as
+    well as cause problematic behavior with handling of StopIteration
+    and StopAsyncIteration.
 
     """
-    assert currently_ki_protected()
-    with open_cancel_scope() as scope:
-        nursery = Nursery(current_task(), scope)
-        nested_child_exc = None
-        try:
-            await yield_(nursery)
-        except BaseException as exc:
-            nested_child_exc = exc
-        assert currently_ki_protected()
-        await nursery._nested_child_finished(nested_child_exc)
+
+    @enable_ki_protection
+    async def __aenter__(self):
+        self._scope = CancelScope._create(deadline=inf, shield=False)
+        self._nursery = Nursery(current_task(), self._scope)
+        return self._nursery
+
+    @enable_ki_protection
+    async def __aexit__(self, etype, exc, tb):
+        new_exc = await self._nursery._nested_child_finished(exc)
+        # Tracebacks show the 'raise' line below out of context, so let's give
+        # this variable a name that makes sense out of context.
+        combined_error_from_nursery = self._scope._close(new_exc)
+        if combined_error_from_nursery is None:
+            return True
+        elif combined_error_from_nursery is exc:
+            return False
+        else:
+            # Copied verbatim from MultiErrorCatcher.  Python doesn't
+            # allow us to encapsulate this __context__ fixup.
+            old_context = combined_error_from_nursery.__context__
+            try:
+                raise combined_error_from_nursery
+            finally:
+                _, value, _ = sys.exc_info()
+                assert value is combined_error_from_nursery
+                value.__context__ = old_context
+
+    def __enter__(self):
+        raise RuntimeError(
+            "use 'async with open_nursery(...)', not 'with open_nursery(...)'"
+        )
+
+    def __exit__(self):  # pragma: no cover
+        assert False, """Never called, but should be defined"""
 
 
-# I *think* this is equivalent to the above, and it gives *much* nicer
-# exception tracebacks... but I'm a little nervous about it because it's much
-# trickier code :-(
-#
-# class NurseryManager:
-#     @enable_ki_protection
-#     async def __aenter__(self):
-#         self._scope_manager = open_cancel_scope()
-#         scope = self._scope_manager.__enter__()
-#         self._parent_nursery = Nursery(current_task(), scope)
-#         return self._parent_nursery
-#
-#     @enable_ki_protection
-#     async def __aexit__(self, etype, exc, tb):
-#         try:
-#             await self._parent_nursery._clean_up(exc)
-#         except BaseException as new_exc:
-#             if not self._scope_manager.__exit__(
-#                     type(new_exc), new_exc, new_exc.__traceback__):
-#                 if exc is new_exc:
-#                     return False
-#                 else:
-#                     raise
-#         else:
-#             self._scope_manager.__exit__(None, None, None)
-#             return True
-#
-# def open_nursery():
-#     return NurseryManager()
+def open_nursery():
+    return NurseryManager()
 
 
 class Nursery:
@@ -391,13 +454,14 @@ class Nursery:
                 self._parent_waiting_in_aexit = False
                 GLOBAL_RUN_CONTEXT.runner.reschedule(self._parent_task)
 
-    def _child_finished(self, task, result):
+    def _child_finished(self, task, outcome):
         self._children.remove(task)
-        if type(result) is Error:
-            self._add_exc(result.error)
+        if type(outcome) is Error:
+            self._add_exc(outcome.error)
         self._check_nursery_closed()
 
     async def _nested_child_finished(self, nested_child_exc):
+        """Returns MultiError instance if there are pending exceptions."""
         if nested_child_exc is not None:
             self._add_exc(nested_child_exc)
         self._nested_child_running = False
@@ -408,7 +472,7 @@ class Nursery:
             # KeyboardInterrupt), then save that, but still wait until our
             # children finish.
             def aborted(raise_cancel):
-                self._add_exc(outcome.capture(raise_cancel).error)
+                self._add_exc(capture(raise_cancel).error)
                 return Abort.FAILED
 
             self._parent_waiting_in_aexit = True
@@ -425,7 +489,7 @@ class Nursery:
         popped = self._parent_task._child_nurseries.pop()
         assert popped is self
         if self._pending_excs:
-            raise MultiError(self._pending_excs)
+            return MultiError(self._pending_excs)
 
     def start_soon(self, async_fn, *args, name=None):
         GLOBAL_RUN_CONTEXT.runner.spawn_impl(async_fn, args, self, name)
@@ -486,18 +550,16 @@ class Task:
 
     # Invariant:
     # - for unscheduled tasks, _next_send is None
-    # - for scheduled tasks, _next_send is a Result object
+    # - for scheduled tasks, _next_send is an Outcome object,
+    #   and custom_sleep_data is None
     # Tasks start out unscheduled.
     _next_send = attr.ib(default=None)
-    # ParkingLot modifies this directly
     _abort_func = attr.ib(default=None)
+    custom_sleep_data = attr.ib(default=None)
 
     # For introspection and nursery.start()
     _child_nurseries = attr.ib(default=attr.Factory(list))
 
-    # Task-local values, see _local.py
-    _locals = attr.ib(default=attr.Factory(dict))
-
     # these are counts of how many cancel/schedule points this task has
     # executed, for assert{_no,}_yields
     # XX maybe these should be exposed as part of a statistics() method?
@@ -548,7 +610,7 @@ class Task:
         # whether we succeeded or failed.
         self._abort_func = None
         if success is Abort.SUCCEEDED:
-            self._runner.reschedule(self, outcome.capture(raise_cancel))
+            self._runner.reschedule(self, capture(raise_cancel))
 
     def _attempt_delivery_of_any_pending_cancel(self):
         if self._abort_func is None:
@@ -608,11 +670,10 @@ class Runner:
     deadlines = attr.ib(default=attr.Factory(SortedDict))
 
     init_task = attr.ib(default=None)
-    init_task_result = attr.ib(default=None)
     system_nursery = attr.ib(default=None)
     system_context = attr.ib(default=None)
     main_task = attr.ib(default=None)
-    main_task_result = attr.ib(default=None)
+    main_task_outcome = attr.ib(default=None)
 
     entry_queue = attr.ib(default=attr.Factory(EntryQueue))
     trio_token = attr.ib(default=None)
@@ -622,7 +683,8 @@ class Runner:
     def close(self):
         self.io_manager.close()
         self.entry_queue.close()
-        self.instrument("after_run")
+        if self.instruments:
+            self.instrument("after_run")
 
     # Methods marked with @_public get converted into functions exported by
     # trio.hazmat:
@@ -722,8 +784,10 @@ class Runner:
         assert task._next_send is None
         task._next_send = next_send
         task._abort_func = None
+        task.custom_sleep_data = None
         self.runq.append(task)
-        self.instrument("task_scheduled", task)
+        if self.instruments:
+            self.instrument("task_scheduled", task)
 
     def spawn_impl(self, async_fn, args, nursery, name, *, system_task=False):
 
@@ -798,7 +862,7 @@ class Runner:
         # We can't check iscoroutinefunction(async_fn), because that will fail
         # for things like functools.partial objects wrapping an async
         # function. So we have to just call it and then check whether the
-        # result is a coroutine object.
+        # return value is a coroutine object.
         if not isinstance(coro, collections.abc.Coroutine):
             # Give good error for: nursery.start_soon(func_returning_future)
             if _return_value_looks_like_wrong_library(coro):
@@ -858,40 +922,35 @@ class Runner:
             for scope in nursery._cancel_stack:
                 scope._add_task(task)
 
-            # Task locals are inherited from the spawning task, not the
-            # nursery task. The 'if nursery' check is just used as a guard to
-            # make sure we don't try to do this to the root task.
-            parent_task = current_task()
-            for local, values in parent_task._locals.items():
-                task._locals[local] = dict(values)
-
-        self.instrument("task_spawned", task)
-        # Special case: normally next_send should be a Result, but for the
+        if self.instruments:
+            self.instrument("task_spawned", task)
+        # Special case: normally next_send should be an Outcome, but for the
         # very first send we have to send a literal unboxed None.
         self.reschedule(task, None)
         return task
 
-    def task_exited(self, task, result):
+    def task_exited(self, task, outcome):
         while task._cancel_stack:
             task._cancel_stack[-1]._remove_task(task)
         self.tasks.remove(task)
-        if task._parent_nursery is None:
+        if task is self.main_task:
+            self.main_task_outcome = outcome
+            self.system_nursery.cancel_scope.cancel()
+            self.system_nursery._child_finished(task, Value(None))
+        elif task is self.init_task:
+            # If the init task crashed, then something is very wrong and we
+            # let the error propagate. (It'll eventually be wrapped in a
+            # TrioInternalError.)
+            outcome.unwrap()
             # the init task should be the last task to exit. If not, then
-            # something is very wrong. Probably it hit some unexpected error,
-            # in which case we re-raise the error (which will later get
-            # converted to a TrioInternalError, but at least we'll get a
-            # traceback). Otherwise, raise a new error.
+            # something is very wrong.
             if self.tasks:  # pragma: no cover
-                result.unwrap()
                 raise TrioInternalError
         else:
-            task._parent_nursery._child_finished(task, result)
-        if task is self.main_task:
-            self.main_task_result = result
-            self.system_nursery.cancel_scope.cancel()
-        if task is self.init_task:
-            self.init_task_result = result
-        self.instrument("task_exited", task)
+            task._parent_nursery._child_finished(task, outcome)
+
+        if self.instruments:
+            self.instrument("task_exited", task)
 
     ################
     # System tasks and init
@@ -915,7 +974,10 @@ class Runner:
 
         * By default, system tasks have :exc:`KeyboardInterrupt` protection
           *enabled*. If you want your task to be interruptible by control-C,
-          then you need to use :func:`disable_ki_protection` explicitly.
+          then you need to use :func:`disable_ki_protection` explicitly (and
+          come up with some plan for what to do with a
+          :exc:`KeyboardInterrupt`, given that system tasks aren't allowed to
+          raise exceptions).
 
         * System tasks do not inherit context variables from their creator.
 
@@ -935,40 +997,21 @@ class Runner:
 
         """
 
-        async def system_task_wrapper(async_fn, args):
-            PASS = (
-                Cancelled, KeyboardInterrupt, GeneratorExit, TrioInternalError
-            )
-
-            def excfilter(exc):
-                if isinstance(exc, PASS):
-                    return exc
-                else:
-                    new_exc = TrioInternalError("system task crashed")
-                    new_exc.__cause__ = exc
-                    return new_exc
-
-            with MultiError.catch(excfilter):
-                await async_fn(*args)
-
-        if name is None:
-            name = async_fn
         return self.spawn_impl(
-            system_task_wrapper,
-            (async_fn, args),
-            self.system_nursery,
-            name,
-            system_task=True,
+            async_fn, args, self.system_nursery, name, system_task=True
         )
 
     async def init(self, async_fn, args):
         async with open_nursery() as system_nursery:
             self.system_nursery = system_nursery
-            self.main_task = self.spawn_impl(
-                async_fn, args, system_nursery, None
-            )
+            try:
+                self.main_task = self.spawn_impl(
+                    async_fn, args, system_nursery, None
+                )
+            except BaseException as exc:
+                self.main_task_outcome = Error(exc)
+                system_nursery.cancel_scope.cancel()
             self.entry_queue.spawn()
-        return self.main_task_result.unwrap()
 
     ################
     # Outside context problems
@@ -1011,7 +1054,7 @@ class Runner:
         # the same time -- so even if KI arrives before main_task is created,
         # we won't get here until afterwards.
         assert self.main_task is not None
-        if self.main_task_result is not None:
+        if self.main_task_outcome is not None:
             # We're already in the process of exiting -- leave ki_pending set
             # and we'll check it again on our way out of run().
             return
@@ -1099,6 +1142,9 @@ class Runner:
     ################
 
     def instrument(self, method_name, *args):
+        if not self.instruments:
+            return
+
         for instrument in list(self.instruments):
             try:
                 method = getattr(instrument, method_name)
@@ -1160,7 +1206,7 @@ def run(
     instruments=(),
     restrict_keyboard_interrupt_to_checkpoints=False
 ):
-    """Run a trio-flavored async function, and return the result.
+    """Run a trio-flavored async function, and return the outcome.
 
     Calling::
 
@@ -1213,7 +1259,7 @@ def run(
           This setting has no effect if your program has registered a custom
           SIGINT handler, or if :func:`run` is called from anywhere but the
           main thread (this is a Python limitation), or if you use
-          :func:`catch_signals` to catch SIGINT.
+          :func:`open_signal_receiver` to catch SIGINT.
 
     Returns:
       Whatever ``async_fn`` returns.
@@ -1265,7 +1311,7 @@ def run(
                 with closing(runner):
                     # The main reason this is split off into its own function
                     # is just to get rid of this extra indentation.
-                    result = run_impl(runner, async_fn, args)
+                    run_impl(runner, async_fn, args)
             except TrioInternalError:
                 raise
             except BaseException as exc:
@@ -1274,12 +1320,17 @@ def run(
                 ) from exc
             finally:
                 GLOBAL_RUN_CONTEXT.__dict__.clear()
-            return result.unwrap()
+            # Inlined copy of runner.main_task_outcome.unwrap() to avoid
+            # cluttering every single trio traceback with an extra frame.
+            if type(runner.main_task_outcome) is Value:
+                return runner.main_task_outcome.value
+            else:
+                raise runner.main_task_outcome.error
     finally:
         # To guarantee that we never swallow a KeyboardInterrupt, we have to
         # check for pending ones once more after leaving the context manager:
         if runner.ki_pending:
-            # Implicitly chains with any exception from result.unwrap():
+            # Implicitly chains with any exception from outcome.unwrap():
             raise KeyboardInterrupt
 
 
@@ -1291,7 +1342,8 @@ _MAX_TIMEOUT = 24 * 60 * 60
 def run_impl(runner, async_fn, args):
     __tracebackhide__ = True
 
-    runner.instrument("before_run")
+    if runner.instruments:
+        runner.instrument("before_run")
     runner.clock.start_clock()
     runner.init_task = runner.spawn_impl(
         runner.init,
@@ -1320,9 +1372,13 @@ def run_impl(runner, async_fn, args):
                 timeout = cushion
                 idle_primed = True
 
-        runner.instrument("before_io_wait", timeout)
+        if runner.instruments:
+            runner.instrument("before_io_wait", timeout)
+
         runner.io_manager.handle_io(timeout)
-        runner.instrument("after_io_wait", timeout)
+
+        if runner.instruments:
+            runner.instrument("after_io_wait", timeout)
 
         now = runner.clock.current_time()
         # We process all timeouts in a batch and then notify tasks at the end
@@ -1371,30 +1427,39 @@ def run_impl(runner, async_fn, args):
         while batch:
             task = batch.pop()
             GLOBAL_RUN_CONTEXT.task = task
-            runner.instrument("before_task_step", task)
+
+            if runner.instruments:
+                runner.instrument("before_task_step", task)
 
             next_send = task._next_send
             task._next_send = None
-            final_result = None
+            final_outcome = None
             try:
-                # We used to unwrap the Result object here and send/throw its
+                # We used to unwrap the Outcome object here and send/throw its
                 # contents in directly, but it turns out that .throw() is
                 # buggy, at least on CPython 3.6 and earlier:
                 #   https://bugs.python.org/issue29587
                 #   https://bugs.python.org/issue29590
-                # So now we send in the Result object and unwrap it on the
+                # So now we send in the Outcome object and unwrap it on the
                 # other side.
                 msg = task.context.run(task.coro.send, next_send)
             except StopIteration as stop_iteration:
-                final_result = Value(stop_iteration.value)
+                final_outcome = Value(stop_iteration.value)
             except BaseException as task_exc:
-                final_result = Error(task_exc)
+                # Store for later, removing uninteresting top frames: 1 frame
+                # we always remove, because it's this function catching it,
+                # and then in addition we remove however many more Context.run
+                # adds.
+                tb = task_exc.__traceback__.tb_next
+                for _ in range(CONTEXT_RUN_TB_FRAMES):
+                    tb = tb.tb_next
+                final_outcome = Error(task_exc.with_traceback(tb))
 
-            if final_result is not None:
+            if final_outcome is not None:
                 # We can't call this directly inside the except: blocks above,
                 # because then the exceptions end up attaching themselves to
                 # other exceptions as __context__ in unwanted ways.
-                runner.task_exited(task, final_result)
+                runner.task_exited(task, final_outcome)
             else:
                 task._schedule_points += 1
                 if msg is CancelShieldedCheckpoint:
@@ -1425,11 +1490,10 @@ def run_impl(runner, async_fn, args):
                     # and propagate the exception into the task's spawner.
                     runner.task_exited(task, Error(exc))
 
-            runner.instrument("after_task_step", task)
+            if runner.instruments:
+                runner.instrument("after_task_step", task)
             del GLOBAL_RUN_CONTEXT.task
 
-    return runner.init_task_result
-
 
 ################################################################
 # Other public API functions
